using System;
using System.Collections.Generic;
using System.Text;

namespace MIDI2ASQ
{
    class ASQWriter
    {
        const int offset = 21;
        const int div = 250;

        List<ASQChannal> channels;
        long ticks;
        public ASQWriter()
        {
            channels = new List<ASQChannal>();
        }

        public void Start()
        {
            ticks = DateTime.Now.Ticks;
        }

        public void NoteOn(int i, int channel)
        {
            int ch = FreeChannel();
            channels[ch].Add(new ASQTone(0, 128, (DateTime.Now.Ticks - channels[ch].ticks) / div, -1));
            channels[ch].Add(new ASQTone((byte)(i - offset), 255, DateTime.Now.Ticks, channel));
            channels[ch].Free = false;
        }
        public void NoteOff(int x, int channel)
        {
            for (int i = 0; i < channels.Count; i++)
            {
                if (channels[i].Last.channel == channel && channels[i].Last.note == x - offset)
                {
                    channels[i].Last.time = (DateTime.Now.Ticks - channels[i].Last.time) / div;
                    channels[i].Free = true;
                    return;
                }
            }
        }
        private int FreeChannel()
        {
            for (int i = 0; i < channels.Count; i++)
                if (channels[i].Free)
                    return i;
            channels.Add(new ASQChannal());
            channels[channels.Count - 1].Add(new ASQTone(0, 128, (DateTime.Now.Ticks - ticks) / div, -1));
            return channels.Count - 1;
        }
        public override string ToString()
        {
            string s = "";
            for (int i = 0; i < channels.Count; i++)
                s += "Channel" + i + ":\r\n" + channels[i].ToString();
            return s;
        }

        public string GetCode()
        {
            string code = "";
            for (int i = 0; i < channels.Count; i++)
            {
                code += channels[i].GetCode(i);
            }
            code += "\r\nStream stream["+channels.Count+"];\r\n";
            for (int i = 0; i < channels.Count; i++)
            {
                code += "stream[" + i + "] = CreateStream(tone" + i + ", " + channels[i].tones.Count.ToString().PadLeft(3) + ");\r\n";
                code += "#ifdef PROGRESS\r\nstd::cout << \"Progress: " + i + " / " + channels.Count + "\n\";";
            }
            code += "\r\nPolyStream polystream = CreatePolyStream(stream, " + Math.Round(Math.Log(channels.Count,2)+0.5,0) + ");\r\n";
            return code;
        }
    }
    class ASQChannal
    {
        public List<ASQTone> tones;
        bool free;
        public long ticks;
        public ASQChannal()
        {
            Free = true;
            tones = new List<ASQTone>();
        }
        public void Add(ASQTone t)
        {
            tones.Add(t);
        }
        public ASQTone Last
        {
            get
            {
                return tones[tones.Count - 1];
            }
        }

        public bool Free
        {
            get
            {
                return free;
            }
            set
            {
                free = value;
                ticks = DateTime.Now.Ticks;
            }
        }

        public override string ToString()
        {
            string s = "";
            for (int i = 0; i < tones.Count; i++)
                s += " - " + tones[i].ToString() + "\r\n";
            return s;
        }
        public string GetCode(int index)
        {
            string code = "Tone tone" + index + "[" + tones.Count + "];\r\n";
            for (int i = 0; i < tones.Count; i++)
            {
                code += "tone" + index + "[" + i + "]" + " = " + tones[i].GetCode() + "\r\n";
            }
            return code + "\r\n";
        }
    }
    class ASQTone
    {
        public byte note, volume;
        public long time, channel;
        public ASQTone(byte note, byte vol, long time, int channel)
        {
            volume = vol;
            this.note = note;
            this.time = time;
            this.channel = channel;
        }

        public override string ToString()
        {
            return "{ Note: " + note + ", Vol: " + volume + ", Time: " + time + " }";
        }

        public string GetCode()
        {
            long t = time;
            if(time >= 10000000)
                t = 0;
            return "CreateTone(" + note.ToString().PadLeft(3) + ", " + volume.ToString().PadLeft(3) + ", " + t.ToString().PadLeft(10) + ");";
        }
    }
}
